@@ -1,7 +1,6 @@
 from __future__ import absolute_import
 
 import json
-import functools
 import re
 
 try:
@@ -9,10 +8,10 @@ try:
 except ImportError:
     from urllib import urlencode
 
+import tornado.gen
 import tornado.web
 import tornado.auth
 
-from tornado import httpclient
 from tornado.options import options
 from celery.utils.imports import instantiate
 
@@ -22,17 +21,17 @@ from ..views import BaseHandler
 class GoogleAuth2LoginHandler(BaseHandler, tornado.auth.GoogleOAuth2Mixin):
     _OAUTH_SETTINGS_KEY = 'oauth'
 
-    @tornado.web.asynchronous
+    @tornado.gen.coroutine
     def get(self):
         redirect_uri = self.settings[self._OAUTH_SETTINGS_KEY]['redirect_uri']
         if self.get_argument('code', False):
-            self.get_authenticated_user(
+            user = yield self.get_authenticated_user(
                 redirect_uri=redirect_uri,
                 code=self.get_argument('code'),
-                callback=self._on_auth,
             )
+            yield self._on_auth(user)
         else:
-            self.authorize_redirect(
+            yield self.authorize_redirect(
                 redirect_uri=redirect_uri,
                 client_id=self.settings[self._OAUTH_SETTINGS_KEY]['key'],
                 scope=['profile', 'email'],
@@ -40,19 +39,20 @@ class GoogleAuth2LoginHandler(BaseHandler, tornado.auth.GoogleOAuth2Mixin):
                 extra_params={'approval_prompt': 'auto'}
             )
 
+    @tornado.gen.coroutine
     def _on_auth(self, user):
         if not user:
             raise tornado.web.HTTPError(403, 'Google auth failed')
         access_token = user['access_token']
 
         try:
-            response = httpclient.HTTPClient().fetch(
-                'https://www.googleapis.com/plus/v1/people/me',
+            response = yield self.get_auth_http_client().fetch(
+                'https://www.googleapis.com/userinfo/v2/me',
                 headers={'Authorization': 'Bearer %s' % access_token})
         except Exception as e:
             raise tornado.web.HTTPError(403, 'Google auth failed: %s' % e)
 
-        email = json.loads(response.body.decode('utf-8'))['emails'][0]['value']
+        email = json.loads(response.body.decode('utf-8'))['email']
         if not re.match(self.application.options.auth, email):
             message = (
                 "Access denied to '{email}'. Please use another account or "
@@ -62,8 +62,11 @@ class GoogleAuth2LoginHandler(BaseHandler, tornado.auth.GoogleOAuth2Mixin):
 
         self.set_secure_cookie("user", str(email))
 
-        next = self.get_argument('next', '/')
-        self.redirect(next)
+        next_ = self.get_argument('next', self.application.options.url_prefix or '/')
+        if self.application.options.url_prefix and next[0] != '/':
+            next_ = '/' + next_
+
+        self.redirect(next_)
 
 
 class LoginHandler(BaseHandler):
@@ -78,9 +81,8 @@ class GithubLoginHandler(BaseHandler, tornado.auth.OAuth2Mixin):
     _OAUTH_NO_CALLBACKS = False
     _OAUTH_SETTINGS_KEY = 'oauth'
 
-    @tornado.auth._auth_return_future
-    def get_authenticated_user(self, redirect_uri, code, callback):
-        http = self.get_auth_http_client()
+    @tornado.gen.coroutine
+    def get_authenticated_user(self, redirect_uri, code):
         body = urlencode({
             "redirect_uri": redirect_uri,
             "code": code,
@@ -89,36 +91,29 @@ class GithubLoginHandler(BaseHandler, tornado.auth.OAuth2Mixin):
             "grant_type": "authorization_code",
         })
 
-        http.fetch(
+        response = yield self.get_auth_http_client().fetch(
             self._OAUTH_ACCESS_TOKEN_URL,
-            functools.partial(self._on_access_token, callback),
             method="POST",
             headers={'Content-Type': 'application/x-www-form-urlencoded',
                      'Accept': 'application/json'}, body=body)
 
-    @tornado.web.asynchronous
-    def _on_access_token(self, future, response):
         if response.error:
-            future.set_exception(tornado.auth.AuthError(
-                'OAuth authentication error: %s' % str(response)))
-            return
+            raise tornado.auth.AuthError(
+                'OAuth authenticator error: %s' % str(response))
 
-        future.set_result(json.loads(response.body.decode('utf-8')))
+        raise tornado.gen.Return(json.loads(response.body.decode('utf-8')))
 
-    def get_auth_http_client(self):
-        return httpclient.AsyncHTTPClient()
-
-    @tornado.web.asynchronous
+    @tornado.gen.coroutine
     def get(self):
         redirect_uri = self.settings[self._OAUTH_SETTINGS_KEY]['redirect_uri']
         if self.get_argument('code', False):
-            self.get_authenticated_user(
+            user = yield self.get_authenticated_user(
                 redirect_uri=redirect_uri,
                 code=self.get_argument('code'),
-                callback=self._on_auth,
             )
+            yield self._on_auth(user)
         else:
-            self.authorize_redirect(
+            yield self.authorize_redirect(
                 redirect_uri=redirect_uri,
                 client_id=self.settings[self._OAUTH_SETTINGS_KEY]['key'],
                 scope=['user:email'],
@@ -126,17 +121,17 @@ class GithubLoginHandler(BaseHandler, tornado.auth.OAuth2Mixin):
                 extra_params={'approval_prompt': 'auto'}
             )
 
-    @tornado.web.asynchronous
+    @tornado.gen.coroutine
     def _on_auth(self, user):
         if not user:
             raise tornado.web.HTTPError(500, 'OAuth authentication failed')
         access_token = user['access_token']
 
-        req = httpclient.HTTPRequest(
+        response = yield self.get_auth_http_client().fetch(
             'https://api.github.com/user/emails',
             headers={'Authorization': 'token ' + access_token,
                      'User-agent': 'Tornado auth'})
-        response = httpclient.HTTPClient().fetch(req)
+
 
         emails = [email['email'].lower() for email in json.loads(response.body.decode('utf-8'))
                   if email['verified'] and re.match(self.application.options.auth, email['email'])]
@@ -150,7 +145,9 @@ class GithubLoginHandler(BaseHandler, tornado.auth.OAuth2Mixin):
 
         self.set_secure_cookie("user", str(emails.pop()))
 
-        next_ = self.get_argument('next', '/')
+        next_ = self.get_argument('next', self.application.options.url_prefix or '/')
+        if self.application.options.url_prefix and next_[0] != '/':
+            next_ = '/' + next_
         self.redirect(next_)
 
 
